class __declspec(dllexport) attr_list;
class __declspec(dllexport) TranslatorWP;
__declspec(dllexport) int sort_list(const void *first, const void *second);

#include "TransWP.h"

//----------------List functions------------------------------------------

class attr_list : public BList { 
	friend class TranslatorWP; //------------No one but TransWP needs to see this-------
	friend int sort_list(const void *first, const void *second);
	protected:
		//
		typedef struct {
			int32 offset;
			WP_Attr *attr;
		} attr_offset;
		//
		attr_list(void) : BList() {}
		bool AddItem(int32 offset, WP_Attr *attr) {
			attr_offset *to_add = new attr_offset;
			to_add->offset = offset; to_add->attr = attr;
			bool result = BList::AddItem(to_add);
			SortItems(&sort_list);
			return result;
		}
		int32 OffsetOf(WP_Attr *attr) {
			attr_offset *cur;
			for (int32 i = 0; i < CountItems(); i++) {
				cur = (attr_offset *)BList::ItemAt(i);
				if (cur->attr == attr)
					return cur->offset;
			}
			return -1;
		}
		int32 IndexOfOffset(int32 offset) {
			attr_offset *cur;
			for (int32 i = 0; i < CountItems(); i++) {
				cur = (attr_offset *)BList::ItemAt(i);
				if (cur->offset == offset)
					return i;
			}
			return -1;
		}
		WP_Attr *ItemAt(int32 offset,int32 index) {
			int32 ind = IndexOfOffset(offset);
			ind += index;
			attr_offset *attr = (attr_offset *)BList::ItemAt(ind);
			if (attr == NULL)
				return NULL;
			if (attr->offset != offset)
				return NULL;
			return attr->attr;
		}
};

int sort_list(const void *first, const void *second) {
	attr_list::attr_offset *one, *two;
	if (one->offset < two->offset)
		return -1;
	if (one->offset > two->offset)
		return 1;
	return 0;
}

//----------------------Begin TransWP Functions-----------------------------
TranslatorWP::TranslatorWP(void) : BArchivable() {
	begin_attrs = new attr_list;
	end_attrs = new attr_list;
}

TranslatorWP::TranslatorWP(const char *plainText) : BArchivable() {
	begin_attrs = new attr_list;
	end_attrs = new attr_list;
	SetText(plainText);
}

TranslatorWP::TranslatorWP(BMessage *archive) : BArchivable(archive) {
	begin_attrs = new attr_list;
	end_attrs = new attr_list;
	SetText(archive->FindString("text"));
	int32 count;
	type_code type;
	archive->GetInfo("attr",&type,&count);
	//----Stuff we'll need-------------------
	const void* buffer;
	uint32 attr_type;
	int32 begin,end;
	ssize_t size;
	size_t data_size;
	//---------------------------------------
	for (int32 i = 0; i < count; i++) {
		archive->FindData("attr",type,i,&buffer,&size);
		memcpy(&begin,buffer,4);
		memcpy(&end,(void *)((uint32)buffer + 4),4);
		memcpy(&attr_type,(void *)((uint32)buffer + 8),4);
		memcpy(&data_size,(void *)((uint32)buffer + 12),sizeof(size_t));
		pushAttr(begin,end,attr_type,(void *)((uint32)buffer + 12 + sizeof(size_t)),data_size);
	}
}

void TranslatorWP::SetText(const char *plainText) {
	text.SetTo(plainText);
}

const char *TranslatorWP::Text(void) {
	return text.String();
}

void TranslatorWP::pushAttr(int32 begin, int32 end, uint32 type, const void *data, size_t size) {
	WP_Attr *attr = new WP_Attr;
	attr->attr_type = type; attr->size = size;
	attr->data = new unsigned char[size];
	memcpy((void *)(attr->data),(void *)data,size);
	begin_attrs->AddItem(begin,attr);
	end_attrs->AddItem(end,attr);
}

WP_Attr *TranslatorWP::getAttr(int32 begin, int32 *end, int32 index) {
	WP_Attr *attr = begin_attrs->ItemAt(begin,index);
	*end = end_attrs->OffsetOf(attr);
	return attr;
}

WP_Attr *TranslatorWP::getAttr(int32 *begin, int32 end, int32 index) {
	WP_Attr *attr = end_attrs->ItemAt(end,index);
	*begin = begin_attrs->OffsetOf(attr);
	return attr;
}

status_t TranslatorWP::Archive(BMessage *archive,bool deep) {
	status_t result = BArchivable::Archive(archive,deep);
	if (result != B_OK)
		return result;
	int32 count = begin_attrs->CountItems();
	archive->AddString("text",text.String());
	//----Stuff we'll need-------------------
	unsigned char* buffer;
	WP_Attr *c;
	int32 d;
	attr_list::attr_offset *b;
	//---------------------------------------
	for (int32 i = 0; i < count; i++) {
		b = (attr_list::attr_offset *)begin_attrs->BList::ItemAt(i);
		c = b->attr;
		buffer = new unsigned char[12 + sizeof(size_t) + c->size];
		d = end_attrs->OffsetOf(c);
		memcpy(buffer,&(b->offset),4);
		memcpy(buffer + 4,&d,4);
		memcpy(buffer + 8,&(c->attr_type),4);
		memcpy(buffer + 12,&(c->size),sizeof(size_t));
		memcpy(buffer + 12 + sizeof(size_t),c->data,c->size);
		archive->AddData("attr",'WPTR',buffer,12 + sizeof(size_t) + c->size,false);
		delete buffer;
	}
	return B_OK;
}

BArchivable *TranslatorWP::Instantiate(BMessage *archive) {
	if ( !validate_instantiation(archive, "TranslatorWP") ) 
		return NULL; 
	return new TranslatorWP(archive);
}

TranslatorWP::~TranslatorWP(void) {
	attr_list::attr_offset *b;
	for (int32 i = 0; i < begin_attrs->CountItems(); i++) {
		b = (attr_list::attr_offset *)begin_attrs->BList::ItemAt(i);
		delete b->attr->data;
		delete b->attr;
		delete b;
	}
	delete begin_attrs;
	delete end_attrs;
}